@@ -18,6 +18,9 @@
 
 #include "common.h"
 #include <errno.h>
+#if HAVE_GETTEXT
+# include <libintl.h>
+#endif
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -25,9 +28,6 @@
 #include <sys/stat.h>
 #include <time.h>
 #include <wchar.h>
-#if HAVE_GETTEXT
-# include <libintl.h>
-#endif
 #include "expand.h"
 #include "hashtable.h"
 #include "mail.h"
@@ -134,20 +134,22 @@ bool is_time_to_check_mail(void)
 void check_mail_and_print_message(void)
 {
     /* Firstly, check the $MAILPATH variable */
-    struct get_variable mailpath = get_variable(L VAR_MAILPATH);
+    struct get_variable_T mailpath = get_variable(L VAR_MAILPATH);
     switch (mailpath.type) {
 	case GV_NOTFOUND:
 	    break;
 	case GV_SCALAR:
 	    activate();
 	    handle_mailpath(mailpath.values[0]);
-	    plfree(mailpath.values, free);
-	    return;
+	    goto mailpath_handled;
 	case GV_ARRAY:
 	case GV_ARRAY_CONCAT:
 	    activate();
 	    for (size_t i = 0; i < mailpath.count; i++)
 		handle_mailpath_element(mailpath.values[i]);
+mailpath_handled:
+	    if (mailpath.freevalues)
+		plfree(mailpath.values, free);
 	    return;
     }
 
