@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* path.h: filename-related utilities */
-/* (C) 2007-2010 magicant */
+/* (C) 2007-2011 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +22,8 @@
 #include <stddef.h>
 #include <sys/types.h>
 
+struct stat;
+
 
 extern _Bool is_file(const char *path)
     __attribute__((nonnull));
@@ -42,6 +44,9 @@ extern _Bool is_executable_regular(const char *path)
 extern _Bool is_directory(const char *path)
     __attribute__((nonnull));
 
+extern _Bool stat_result_same_file(
+	const struct stat *stat1, const struct stat *stat2)
+    __attribute__((nonnull,pure));
 extern _Bool is_same_file(const char *path1, const char *path2)
     __attribute__((nonnull));
 
@@ -84,7 +89,7 @@ extern const wchar_t *get_home_directory(
 
 /********** wglob **********/
 
-enum wglbflags {
+enum wglobflags_T {
     WGLB_MARK     = 1 << 0,
     WGLB_CASEFOLD = 1 << 1,
     WGLB_PERIOD   = 1 << 2,
@@ -94,12 +99,12 @@ enum wglbflags {
 
 struct plist_T;
 
-extern _Bool wglob(const wchar_t *restrict pattern, enum wglbflags flags,
+extern _Bool wglob(const wchar_t *restrict pattern, enum wglobflags_T flags,
 	struct plist_T *restrict list)
     __attribute__((nonnull));
 
 
-/********** Builtins **********/
+/********** Built-ins **********/
 
 extern int cd_builtin(int argc, void **argv)
     __attribute__((nonnull));
